!=======================================================================
      subroutine legendre(n,gl,wgt,ws,level)
!=======================================================================
      implicit none
      integer n,level
      real*8  gl(*),wgt(*),ws(n+1,*)

      integer j,k,id1,id2,np1
      real*8  alpha,beta,dk1,dk2,temp
 
      if(n.le.0) stop ' n is too small.'

      alpha=0.d0
      beta =0.d0  ! Gauss-Lobatto points
      np1  =n+1

!-----------------------------------
!  Gauss-Lobatto collocation points
!-----------------------------------

      call jacobl(n,alpha,beta,gl,level)
 
!----------------------
!--- Find Ln(gl(j))
!----------------------

      if(n.eq.1)then
          wgt(1)=1.d0
          wgt(2)=1.d0
          goto 1000
      endif

      do j=1,np1
         ws(j,1) = 1.d0
         ws(j,2) = gl(j)
      enddo

      do k = 2,n
          id1 = mod(k+1,2)+1
          id2 = mod(k,  2)+1
          dk1 = dfloat(2*k-1)/dfloat(k)
          dk2 = dfloat(k-1)/dfloat(k)
          do j = 1,np1
              wgt(j) = dk1*gl(j)*ws(j,id1)-dk2*ws(j,id2)
              ws(j,id2) =  wgt(j)
          enddo
      enddo
 
!----------------------
!--- Find wgt(j)
!----------------------

      temp = 2.0d0/dfloat(n*np1)
      do j = 1,np1
         wgt(j) = temp/(wgt(j)**2)
      enddo
 
!----------------------
!--- Print Out
!----------------------
 1000 if(level.lt.3) return

      temp = 0.0d0
      do j =1,np1
           temp = temp + wgt(j)
      enddo
      print*,"n=",n
      print*,"G-L points on [-1.0,1.0]:"
      print'(8(f10.5))',(gl(j),j=1,np1)
      print*,"(n+1) weights:"
      print'(8(f10.5))',(wgt(j),j=1,np1)
      print*,"Sum=",temp
 
      return
      end


!=======================================================================
      subroutine jacobl(n,alpha,beta,xjac,level)
!=======================================================================
!  Computes the Gauss-Lobatto collocation points for Jacobi polynomials.
!-----------------------------------------------------------------------
!  N:     degree of approximation
!  alpha: parameter in Jacobi weight
!  beta:  parameter in Jacobi weight
!         (Legendre-GL: alpha=beta=0; Chebyshev-GL: alpha=beta=-0.5)
!  xjac:  output array with the Gauss-Lobatto roots.
!         They are oedered from largest(1.0) to smallest(-1.0).
!-----------------------------------------------------------------------
 
      implicit real*8 (a-h,o-z)
      real*8 xjac(1)
      common /numpar/ zero,half,one,two,pi
      common /jacpar/ alp,bet,rv
 
      zero= 0.d0
      one = 1.d0
      two = 2.d0
      half= 0.5d0
      pi  = 4.d0*datan(1.d0)
      alp = alpha
      bet = beta
      rv  = alpha+one

      eps  =1.d-12
      kstop=20
      np   =n+1
 
      xp = one
      xm = -one
      call jacobf(np,pnp1p,pdnp1p,pnp,pdnp,pnm1p,pdnm1,xp)
      call jacobf(np,pnp1m,pdnp1m,pnm,pdnm,pnm1m,pdnm1,xm)
      det = pnp*pnm1m-pnm*pnm1p
      rp  = -pnp1p
      rm  = -pnp1m
      a   = (rp*pnm1m-rm*pnm1p)/det
      b   = (rm*pnp-rp*pnm)/det
 
      xjac(1) = one
      nh = (n+1)/2
 
      dth= pi/dfloat(2*n+1)
      cd = dcos(two*dth)
      sd = dsin(two*dth)
      cs = dcos(dth)
      ss = dsin(dth)
 
      do j=2,nh
          x = cs
          do k=1,kstop
              call jacobf(np,pnp1,pdnp1,pn,pdn,pnm1,pdnm1,x)
	      poly = pnp1+a*pn+b*pnm1
	      pder = pdnp1+a*pdn+b*pdnm1
	      recsum= zero
	      jm =j-1
	      do i=1,jm
	          recsum = recsum + one/(x-xjac(i))
              enddo
	      delx = -poly/(pder-recsum*poly)
	      x = x+delx
	      if(dabs(delx).lt.eps) goto 100
          enddo
 100      continue
          xjac(j) = x
	  cssave = cs*cd-ss*sd
	  ss = cs*sd+ss*cd
	  cs = cssave
      enddo
      xjac(np) = -one
 
      npp = n+2
      do i = 2,nh
	 xjac(npp-i) = -xjac(i)
      enddo
      if(mod(n,2).eq.0) xjac(nh+1)=zero
 
      return
      end

!=======================================================================
      subroutine jacobf(n,poly,pder,polym1,pderm1,polym2,pderm2,x)
!=======================================================================
!  Computes the Jacobi polynomial(poly) and its derivative(pder)
!  of degree "n" at the point "x".
 
      implicit real*8 (a-h,o-z)
      common /numpar/ zero,half,one,two,pi
      common /jacpar/ alp,bet,rv
 
      apb = alp+bet
      poly= one
      pder= zero
      if(n.eq.0) return
      polylst = poly
      pderlst = pder
      poly = rv*x
      pder = rv
      if(n.eq.1) return
      do k=2,n
	  dk = dfloat(k)
	  a1 = two*dk*(dk+apb)*(two*dk+apb-two)
	  a2 = (two*dk+apb-one)*(alp*alp-bet*bet)
	  b3 = (two*dk+apb-two)
	  a3 = b3*(b3+one)*(b3+two)
	  a4 = two*(dk+alp-one)*(dk+bet-one)*(two*dk+apb)
	  polyn = ((a2+a3*x)*poly-a4*polylst)/a1
	  pdern = ((a2+a3*x)*pder-a4*pderlst+a3*poly)/a1
	  psave = polylst
	  pdsave= pderlst
	  polylst = poly
          pderlst = pder
	  poly = polyn
	  pder = pdern
      enddo
 
      polym1 = polylst
      pderm1 = pderlst
      polym2 = psave
      pderm2 = pdsave
 
      return
      end

